# -*- coding: utf-8 -*-
"""
Created on Thu Apr  1 10:35:30 2021

@author: cyril.charignon
"""

def echangeLigne(M, i1, i2):
    """ M[i1] <-> M[i2] """
    M[i1], M[i2] = M[i2], M[i1] # Ne marcherait pas dans numpy




def dilatation(M, i, x):
    """ M[i] <- x*M[i] """
    for j in range(len(M[i])):
        M[i][j] *= x

def transvection(M, i1, i2, x):
    """ M[i1] <- M[i1] + x*M[i2] """
    for j in range(len(M[i1])):
        M[i1][j] += x*M[i2][j]

def annuleColonne(M, i, j):
    """Précondition : M[i][j] == 1
       Effet : Utilise M[i][j] comme pivot pour annuler les autres coeffs
    de la colonne j."""
    
    for i2 in range(len(M)):
        if i2!=i:
            transvection(M, i2, i, -M[i2][j])
    
test=[[1,2,1,4], [1,0,-1,1], [2,1,2,3]]



def cherche_pivot(M, i, j):
    """ Renvoie l'indice de ligne du coeff de plus grande v abs
    dans la colonne j, en dessous de i."""
    
    res=i
    maxi=abs(M[i][j])
    
    for k in range(i+1, len(M)):
        if abs(M[k][j]) > maxi:
            maxi = abs(M[k][j])
            res=k
    return res

def pivot(M):
    """ Applique le pivot de Gauss en ligne sur M.
    On suppose que M est de rang maximal pour simplifier.
    Donc rang(M) == min(len(M), len(M[0]))
    """
    
    r = min(len(M), len(M[0])) #rang de M, donc nb de pivots à trouver
    for j in range(r):
        #Chercher un pivot
        iPivot = cherche_pivot(M, j, j)
        #Le placer à la bonne place (sur la diagonale ici)
        echangeLigne(M, iPivot, j)
        #Le changer en 1
        dilatation(M, j, 1/M[j][j])
        #Lancer annuleColonne
        annuleColonne(M, j, j)
    
    
# Ex 5
# Le produit scalaire canonique sur C([a,b], R) est défini
# par intégrale
def intégrale(f, a, b, N):
    h= (b-a)/N
    S = (f(a)+f(b))/2
    for i in range(1,N):
        S += f(a+i*h)
    return S*h

def ps(f, g, a, b, N=100):
    return intégrale(lambda t : f(t)*g(t), a, b, N)


def matrice_augmentée_du_système(n, a, b, f):
    """ La matrice des ps(X^i, X^j) avec comme second membre
    les ps(f, X^i)"""
    M = [ [0. for j in range(n+2)] for i in range(n+1)]
    
    for i in range(n+1):
        for j in range(n+1):
            M[i][j] = ps(lambda x: x**i, lambda x:x**j, a, b)
        M[i][n+1] = ps(f, lambda x:x**i, a, b )
    
    return M

import numpy as np
test = matrice_augmentée_du_système(3,0,1,np.sin)
pivot(test)
